! This file is part of HiggsBounds
!  -KW
!****************************************************************** 
module interpolate
!******************************************************************
 implicit none
 
 contains

 !******************************************************************       
 subroutine interpolate_1D(Mh1,t1,datcomp,interpol)
 !******************************************************************
 ! finds the value of 'interpol' at Mh1 by interpolating between points in table t1
 ! t1 is associated with a process involving only one Higgs
 ! datcomp indicates whether the predicted or observed experimental result is used
 ! all valid data in table must be >= 0 as
 ! negative numbers label invalid points
 !****************************************************************** 
  use S95tables_type1
  implicit none
  !--------------------------------------input
  double precision :: Mh1
  integer :: datcomp
  type(table1) :: t1
  !-------------------------------------output
  double precision :: interpol
  !-----------------------------------internal
  integer :: i,ilow
  double precision :: Mh1_below
  double precision :: Mhmin,Mhmax,sep
  integer :: nx
  !-------------------------------------------        

  interpol=0.0D0	
					 	 	 
  Mhmin=t1%Mhmin
  Mhmax=t1%Mhmax
	 
  ! check if mass is within mass range of table:
  if(Mh1 .gt. Mhmax) then
   interpol=-1.0D0	
  elseif(Mh1 .lt. Mhmin) then	 
   interpol=-1.0D0	
  else	
   nx=t1%nx
   sep=t1%sep	 
   	 
   ! linear interpolate between points in table
   ilow=int((Mh1-Mhmin)/sep)+1
   i=ilow	 	 
	 
   Mh1_below=dble(i-1)*sep+Mhmin
   
   if((Mh1-Mh1_below).gt.0.0D0)then
    interpol=t1%dat(i,datcomp) +(t1%dat(i+1,datcomp)-t1%dat(i,datcomp))/sep*(Mh1-Mh1_below) 
   else
    interpol=t1%dat(i,datcomp)   
   endif
     		 			  
  endif 

	
 end subroutine interpolate_1D
 
 !******************************************************************
 subroutine interpolate_2D(Mh1,Mh2,t2,datcomp,interpol)
 !******************************************************************
 ! finds the value of 'interpol' by interpolating between points in table t2
 ! t2 is associated with a process involving two Higgs
 ! datcomp indicates whether the predicted or observed experimental result is used
 ! all valid data in table must be >= 0 as
 ! negative numbers label invalid points
 !****************************************************************** 
  use S95tables_type2	
  implicit none
  !--------------------------------------input
  double precision :: Mh1,Mh2
  integer :: datcomp
  type(table2) :: t2
  !-------------------------------------output
  double precision :: interpol
  !-----------------------------------internal
  integer :: ilow,jlow
  integer :: nx1,nx2
  double precision :: Mhmin1,Mhmax1,Mhmin2,Mhmax2,sep1,sep2
  double precision :: rvec(3),testtri
  double precision :: Mh1bit,Mh2bit,c1,c2,c3
  !-------------------------------------------					  					
  Mhmin1=t2%Mhmin1
  Mhmax1=t2%Mhmax1
  Mhmin2=t2%Mhmin2
  Mhmax2=t2%Mhmax2		
	
  rvec(3)=0	      		
	
  !do i=1,nx2
  ! do j=1,nx1	
  !  dat(i,j)=t2%dat(i,j,datcomp)	  	  	  	  	  
  ! enddo
  !enddo
	 	  	  
  interpol=0.0D0
	
  ! check if mass is within mass range of table:
  if(Mh1.lt. Mhmin1)then
    interpol=-1.0D0
  elseif(Mh1.gt. Mhmax1)then
    interpol=-1.0D0
  elseif(Mh2.lt. Mhmin2)then
    interpol=-1.0D0
  elseif(Mh2.gt. Mhmax2)then
    interpol=-1.0D0		
  else

   nx1=t2%nx1
   nx2=t2%nx2
   sep1=t2%sep1
   sep2=t2%sep2		
 
  ! Interpolation: 
  ! points in table are treated as corners of flat triangular surfaces 	
   Mh1bit=(Mh1-Mhmin1)/sep1
   jlow=1+int(Mh1bit)
   Mh1bit=Mh1bit-dble(int(Mh1bit))
	  	 	 	 
   Mh2bit=(Mh2-Mhmin2)/sep2
   ilow=1+int(Mh2bit)
   Mh2bit=Mh2bit-dble(int(Mh2bit))	
  	    	        
   if(Mh1bit**2.0D0+Mh2bit**2.0D0.le.0.0D0)then! exactly on datapoint
      rvec(3) = t2%dat(ilow,jlow,datcomp)      
   elseif((ilow.eq.nx2).or.(jlow.eq.nx1))then     
      if((ilow.eq.nx2).and.(jlow.eq.nx1))then! exactly on datapoint
       rvec(3) = t2%dat(ilow,jlow,datcomp)
      elseif(ilow.eq.nx2)then
       if(t2%dat(ilow,jlow+1,datcomp).lt.0.0D0)then
        rvec(3)=-6.0D0      
       else
        rvec(3)=t2%dat(ilow,jlow,datcomp)+(t2%dat(ilow,jlow+1,datcomp)-t2%dat(ilow,jlow,datcomp))/sep1*Mh1bit  
       endif     
      elseif(jlow.eq.nx1)then
       if(t2%dat(ilow+1,jlow,datcomp).lt.0.0D0)then
        rvec(3)=-6.0D0      
       else
        rvec(3)=t2%dat(ilow,jlow,datcomp)+(t2%dat(ilow+1,jlow,datcomp)-t2%dat(ilow,jlow,datcomp))/sep2*Mh2bit       
       endif            
      endif
     
      !write(*,*)'interpolate_2D special case:',ilow,jlow,nx2,nx1,t2%dat(ilow,jlow,datcomp)       
      !write(*,*)'interpolate_2D special case cont. ',rvec(3)          
   else   
    !            Mh1,j -->  	
    !            *********  testtri will find out which triangle required point r is in
    !        |   * *   B *  i.e. triangle of orientation 'A' or 'B'  
    !        v   * A *   *
    !     Mh2,i  *     * *    
    !            *********  
    !
    !           Mh1bit
    !            <->      	
    !        ^   ********* ilow,jlow+1  
    !  Mh2bit|   * *     *    
    !        v   * r *   *
    !            *     * *    
    ! ilow+1,jlow*********ilow+1,jlow+1   

    testtri=Mh2bit/sqrt(Mh1bit**2.0D0+Mh2bit**2.0D0) 
    if(testtri.gt.0.7071D0)then !triangle orientation A    
     !1/sqrt(2)=0.707107:slight bias towards orientation A, good for diagonal edge of graph

     !check that all of the corners of the triangle are positive	 
     if(t2%dat(ilow+1,jlow,datcomp).lt.0.0D0)then
      rvec(3)=-6.0D0	  	   
     elseif(t2%dat(ilow+1,jlow+1,datcomp).lt.0.0D0)then
      rvec(3)=-6.0D0	   
     elseif(t2%dat(ilow,jlow,datcomp).lt.0.0D0)then
      rvec(3)=-6.0D0 
     else
      ! find 'height' (i.e. S95 value) of each corner of triangle
	   
      !         c3*       
      !           * *          orth sides have length=1
      !rvec(2)^   * r * 
      !       |   *     *      rvec(3) is S95_interpol
      !       v c1*********c2
      !           <->
      !         rvec(1)   
	   
      c1=t2%dat(ilow+1,jlow,datcomp)
      c2=t2%dat(ilow+1,jlow+1,datcomp)
      c3=t2%dat(ilow,jlow,datcomp)
      ! find position of r relative to corner c1	    
      rvec(1)=Mh1bit
      rvec(2)=1.0D0-Mh2bit
      ! find 'height' of triangle at point r      
      rvec(3)=rvec(1)*(c2-c1)+rvec(2)*(c3-c1)+c1
	    	    	    
     endif
    else !triangle orientation B
     ! check that all of the corners of the triangle are positive	 
     if(t2%dat(ilow,jlow+1,datcomp).lt.0.0D0)then
      rvec(3)=-6.0D0	  	   
     elseif(t2%dat(ilow+1,jlow+1,datcomp).lt.0.0D0)then
      rvec(3)=-6.0D0	   
     elseif(t2%dat(ilow,jlow,datcomp).lt.0.0D0)then
      rvec(3)=-6.0D0 
     else
      !             rvec(1) 
      !               <->	
      !       c2*********c1 ^ 
      !           *     *   | rvec(2)
      !             * r *   v
      !               * *    
      !                 *c3  	   
      c1=t2%dat(ilow,jlow+1,datcomp)
      c2=t2%dat(ilow,jlow,datcomp)
      c3=t2%dat(ilow+1,jlow+1,datcomp)
      ! find position of r relative to corner c1	    
      rvec(1)=1.0D0-Mh1bit
      rvec(2)=Mh2bit
      ! find 'height' of triangle at point r	      
      rvec(3)=rvec(1)*(c2-c1)+rvec(2)*(c3-c1)+c1
	    	    	    		   
     endif
    endif  	   	 	  
   endif

   if(rvec(3).lt.0.0D0)then
    interpol = -1.0D0	  
   else
    interpol =rvec(3)
   endif 

  endif 

 end subroutine interpolate_2D
 !*******************************************************	
	
end module interpolate
