! This file is part of HiggsBounds
!  -KW
!******************************************************************
module theo_manip
!******************************************************************
 !use S95tables_type1
 use usefulbits, only : ndat,nH,theo,tevR,tevextras
 
 implicit none

 type(tevextras) :: tevS(1)

 contains

 !******************************************************************
 subroutine complete_theo
 !decides what has to be done to the input and calls the appropriate 
 !subroutines
 !calculates theo(jj)%inTEVrange(i),theo(jj)%inLEPrange(i)
 !******************************************************************
  use usefulbits, only : whichexpt,whichinput,ndat,&
	  &              TEVrange_min,TEVrange_max,LEPrange_min,LEPrange_max
  implicit none
  !--------------------------------------internal
  integer :: i,j,jj
  !----------------------------------------------

  do jj=1,ndat     
   do i=1,nH
! we want to include contributions to fact(j) from Higgs 
! with masses above TEVt1Mhmim-max_tev_deltaMh but less than TEVt1Mhmax+max_tev_deltaMh
    theo(jj)%inTEVrange(i)=(theo(jj)%Mh(i).gt.TEVrange_min) &
                      .and.(theo(jj)%Mh(i).lt.TEVrange_max)

    theo(jj)%inLEPrange(i)=(theo(jj)%Mh(i).gt.LEPrange_min) &
                      .and.(theo(jj)%Mh(i).lt.LEPrange_max)
   enddo
  enddo

  select case(whichinput)	
  case('effC')  
   call theotevR_from_g2
  case('SLHA')
   call g2_from_Br
   call theotevR_from_g2
  end select               
  
  do jj=1,ndat     
   do j=2,nH
    do i=1,j-1	  
      theo(jj)%lep%XS_hjhi_ratio(i,j) = theo(jj)%lep%XS_hjhi_ratio(j,i) 	      
    enddo
   enddo
  enddo  

  call check_dataset
 
  select case(whichexpt)
  case('onlyT','LandT','singH') ! everything which involves Tevatron tables

   call fill_theo_SM ! n.b. there's no LEP SM cross sections at the moment
  
   select case(whichinput)
   case('part','effC','SLHA') ! everything except option 'hadr', where had XS ratios are inputted directly
    call XS_from_tevR
   end select

  end select   
 end subroutine complete_theo

 !****************************************************************** 
 subroutine g2_from_Br
 ! calculates the effective couplings from the branching ratios
 !***************************************************************** 
  use theory_BRfunctions
  use usefulbits, only : theo,g2,nH,ndat
  implicit none
  !--------------------------------------internal
  integer :: i,jj
  double precision :: Mhi,GammaRat  
  !---------------------------------------------
   do jj=1,ndat  
    do i=1,nH
      Mhi= theo(jj)%Mh(i)
     
      if(theo(jj)%inTEVrange(i).or.theo(jj)%inLEPrange(i))then   
       GammaRat=theo(jj)%Gammatot(i)/BRSM_GammaTot(Mhi)

       g2(jj)%hjZZ(i)=theo(jj)%BR_hjZZ(i)/BRSM_HZZ(Mhi)*GammaRat   
       g2(jj)%hjbb(i)=theo(jj)%BR_hjbb(i)/BRSM_Hbb(Mhi)*GammaRat
       g2(jj)%hjWW(i)=theo(jj)%BR_hjWW(i)/BRSM_HWW(Mhi)*GammaRat
       g2(jj)%hjgg(i)=theo(jj)%BR_hjgg(i)/BRSM_Hgg(Mhi)*GammaRat  
       g2(jj)%hjgaga(i)=theo(jj)%BR_hjgaga(i)/BRSM_Hgaga(Mhi)*GammaRat
       g2(jj)%hjtautau(i)=theo(jj)%BR_hjtautau(i)/BRSM_Htautau(Mhi)*GammaRat 
      else 
       g2(jj)%hjZZ(i)=0.0D0  
       g2(jj)%hjbb(i)=0.0D0 
       g2(jj)%hjWW(i)=0.0D0 
       g2(jj)%hjgg(i)=0.0D0 
       g2(jj)%hjgaga(i)=0.0D0 
       g2(jj)%hjtautau(i)=0.0D0 
      endif      
    enddo   
           
    !recall program stops earlier if nh!=3 and whichinput='SLHA'
    !note approximation: requires a unitary Higgs mixing matrix
    g2(jj)%hjhiZ_in=0.0D0
    g2(jj)%hjhiZ_in(2) = g2(jj)%hjZZ(3)                !(1,2)
    g2(jj)%hjhiZ_in(4) = g2(jj)%hjZZ(2)                !(3,1)
    g2(jj)%hjhiZ_in(5) = g2(jj)%hjZZ(1)                !(3,2)
    !other possibility would be to use Br(hj->hiZ) or Br(Z->hjhi) 
    !but often will be missing (i.e. not kinematically allowed)        
   enddo
 end subroutine g2_from_Br

 !****************************************************************** 
 subroutine theotevR_from_g2 
 ! uses the effective couplings contained in g2 to calculate 
 ! partonic cross section ratios, some hadronic cross section ratios
 ! and branching ratios
 !*****************************************************************
  use theory_BRfunctions
  use usefulbits, only : theo,g2,nH,ndat
  implicit none
  !--------------------------------------internal
  integer :: i,jj,j,k 
  double precision :: Mhi,GammaRat
  double precision :: SM_ZZ_contrib_to_VBF,SM_WW_contrib_to_VBF
  !---------------------------------------------

! relative contributuion of WW- and ZZ-fusion to VBF (in LO) for
! p p-bar collisions at SqrtS=1.96 TeV (calcuated by T. Figy with VBF@NLO):
   SM_ZZ_contrib_to_VBF=0.23D0
   SM_WW_contrib_to_VBF=0.77D0

   do jj=1,ndat
    do i=1,nH   
      Mhi=theo(jj)%Mh(i)
        	
      theo(jj)%lep%XS_hjZ_ratio(i)   =   g2(jj)%hjZZ(i) 
      tevR(jj)%bg_hjb(i)             =   g2(jj)%hjbb(i)
      theo(jj)%tev%XS_vbf_ratio(i)   =   g2(jj)%hjWW(i)*SM_WW_contrib_to_VBF &	
                               &       + g2(jj)%hjZZ(i)*SM_ZZ_contrib_to_VBF
	   
      tevR(jj)%qq_hjWp(:,i)      = g2(jj)%hjWW(i)
      tevR(jj)%qq_hjWm(:,i)      = g2(jj)%hjWW(i) 
      tevR(jj)%gg_hj(i)          = g2(jj)%hjgg(i)
      tevR(jj)%qq_hj(5,i)	 = g2(jj)%hjbb(i)	
      tevR(jj)%qq_hjZ(:,i)       = g2(jj)%hjZZ(i)

      if(theo(jj)%inTEVrange(i).or.theo(jj)%inLEPrange(i))then
   
       GammaRat=theo(jj)%Gammatot(i)/BRSM_GammaTot(Mhi)  
             
       if(theo(jj)%Gammatot(i).gt.0.0D0)then
        theo(jj)%BR_hjbb(i)    = g2(jj)%hjbb(i)    *BRSM_Hbb(Mhi)    /GammaRat
        theo(jj)%BR_hjtautau(i)= g2(jj)%hjtautau(i)*BRSM_Htautau(Mhi)/GammaRat
        theo(jj)%BR_hjWW(i)    = g2(jj)%hjWW(i)    *BRSM_HWW(Mhi)    /GammaRat 
        theo(jj)%BR_hjgaga(i)  = g2(jj)%hjgaga(i)  *BRSM_Hgaga(Mhi)  /GammaRat  
       else

        write(*,*)'at jj=',jj,'i=',i
        write(*,*)'total decay width is less than or equal to zero'
        stop'total decay width is less than or equal to zero'
       endif     
      else
        theo(jj)%BR_hjbb(i)    = 0.0D0
        theo(jj)%BR_hjtautau(i)= 0.0D0 
        theo(jj)%BR_hjWW(i)    = 0.0D0 
        theo(jj)%BR_hjgaga(i)  = 0.0D0 
      endif     	 
	
    enddo 

    k=0
    do j=1,nH
     do i=1,j
      k=k+1
      theo(jj)%lep%XS_hjhi_ratio(j,i)=g2(jj)%hjhiZ_in(k)! note only half of XS_hjhi_ratio is filled
     enddo
    enddo  	

   enddo

 end subroutine theotevR_from_g2

 !*****************************************************************
 subroutine check_dataset
 ! checks each parameter point to determine whether the Higgs masses
 ! and branching ratios make sense
 ! Sets theo(jj)%gooddataset accordingly
 !*****************************************************************
  use usefulbits, only : theo,ndat,debug,nH
  implicit none
  !--------------------------------------internal
  integer :: jj
  double precision :: testsumBR
  double precision,allocatable :: testBR(:)
  double precision :: fuzziness
  !---------------------------------------------
  fuzziness = 0.05D0
  !fuzziness = 100.0D0 ; write(*,*)'WARNING: fuzziness factor is far too high'

  allocate(testBR(nH))

  ! testing to see if the dataset is ok
  do jj=1,ndat 
  
   testBR  =            theo(jj)%BR_hjbb              &
           &      +     theo(jj)%BR_hjtautau          &
           &      +     theo(jj)%BR_hjWW              &
           &      +     theo(jj)%BR_hjgaga            &
	   &      + sum(theo(jj)%BR_hjhihi,dim=2)

   testsumBR=maxval(  testBR   ) 

   if(minval(theo(jj)%Mh).le.0.0D0)then
    theo(jj)%gooddataset=.False.
     if(debug)write(*,*) 'warning: negative Higgs mass at jj=',jj,theo(jj)%Mh
   elseif( testsumBR  .gt.   (1.0D0+fuzziness)   )then !i.e. branching ratios for one of the Higgs add up to more than 1+fuzziness
    !theo(jj)%gooddataset=.False. 
    theo(jj)%gooddataset=.True.    
    if(debug)write(*,*) 'warning: sum of BR at jj=',jj,'is',testsumBR,'for Higgs',maxloc(testBR) 
   elseif( testsumBR  .gt.   1.0D0   )then   
    theo(jj)%gooddataset=.True.   
    if(debug)write(*,*) 'warning: sum of BR at jj=',jj,'is',testsumBR,'for Higgs',maxloc(testBR)      
   else
    theo(jj)%gooddataset=.True.      
   endif 
  enddo
  
  deallocate(testBR)
 
 end subroutine check_dataset

 !*****************************************************************
 subroutine fill_theo_SM
 ! fills the Standard Model part of theo
 ! We do this here to save computational time - these  quantities will be
 ! needed a few times in subroutine calcfact_t1, so don't want to calculate them each time
 !************************************************************
  use theory_BRfunctions
  use theory_XS_SM_functions
  use usefulbits, only : theo,ndat

  implicit none
  !--------------------------------------internal
  integer :: i,n
  double precision :: Mhi
  !----------------------------------------------
  
   do n=1,ndat        
    if(theo(n)%gooddataset) then             
    do i=1,nH
  
     Mhi=theo(n)%Mh(i)    
    
     if( .not.theo(n)%inTEVrange(i) )then      
       theo(n)%tev%XS_HW_SM(i) = 0.0D0
       theo(n)%tev%XS_H_SM(i)  = 0.0D0
       theo(n)%tev%XS_HZ_SM(i) = 0.0D0
       theo(n)%tev%XS_vbf_SM(i)= 0.0D0
      
       theo(n)%tev%XS_Hb_SM(i)    = 0.0D0
       theo(n)%tev%XS_Hb_c1_SM(i) = 0.0D0
       theo(n)%tev%XS_Hb_c2_SM(i) = 0.0D0
          
       theo(n)%BR_Hbb_SM(i)     = -1.0D0
       theo(n)%BR_Htautau_SM(i) = -1.0D0
       theo(n)%BR_HWW_SM(i)     = -1.0D0
       theo(n)%BR_Hgaga_SM(i)   = -1.0D0	
       theo(n)%BR_Hgg_SM(i)     = -1.0D0 
       theo(n)%GammaTot_SM(i)   = -1.0D0	 
     else
      
       theo(n)%BR_HWW_SM(i)    = BRSM_HWW(Mhi)     
       theo(n)%BR_Hbb_SM(i)    = BRSM_Hbb(Mhi)
       theo(n)%BR_Htautau_SM(i)= BRSM_Htautau(Mhi)
       theo(n)%BR_Hgaga_SM(i)  = BRSM_Hgaga(Mhi)	 
       theo(n)%BR_Hgg_SM(i)  = BRSM_Hgg(Mhi)	 
       theo(n)%GammaTot_SM(i)  = BRSM_GammaTot(Mhi)	 
	 
       theo(n)%tev%XS_HZ_SM(i) = XS_qq_HZ_SM(Mhi)   
       theo(n)%tev%XS_HW_SM(i) = XS_qq_HW_SM(Mhi)
       theo(n)%tev%XS_H_SM(i)  = XS_gg_H_SM(Mhi)+XS_bb_H_SM(Mhi)  
       theo(n)%tev%XS_vbf_SM(i)= XS_vbf_SM(Mhi)   
           
       theo(n)%tev%XS_Hb_SM(i)    = XS_bg_Hb_SM(Mhi) 
       theo(n)%tev%XS_Hb_c1_SM(i) = XS_bg_Hb_c1_SM(Mhi) 
       theo(n)%tev%XS_Hb_c2_SM(i) = XS_bg_Hb_c2_SM(Mhi)               
     endif  
    enddo
    endif                     
   enddo        
      
 end subroutine fill_theo_SM

 !************************************************************
 subroutine XS_from_tevR
 ! turn partonic cross section ratios in to hadronic cross section
 ! ratios
 ! Subroutine is complicated by the fact that if e.g. all
 ! the tevR(n)%qq_hjW partonic cross section ratios are equal,
 ! just want to use this value for the hadronic cross section
 ! ratio and not lose any accuracy by combining with the tevS
 !************************************************************
  use usefulbits, only : ndat, &
               & allocate_tevextras_parts,deallocate_tevextras_parts

  implicit none
  !--------------------------------------internal
  integer :: n,i
  logical :: simple_tevR
  !----------------------------------------------

    call allocate_tevextras_parts(tevS,nH) 
  
    do n=1,ndat        
     if(theo(n)%gooddataset) then             
      do i=1,nH

       if( theo(n)%inTEVrange(i) )then	 

        call fill_tevS(i,theo(n)%Mh(i))      
        
        !this if loop is here to make sure tevR(n)%qq_hjWp(1,i).eq.0.0D0 is taken care of
        if(tevR(n)%qq_hjWp(1,i).eq.0.0D0)then
         simple_tevR=.False.
        elseif( (( sum(abs( tevR(n)%qq_hjWp(:,i) - tevR(n)%qq_hjWp(1,i)))   &
    &            + sum(abs( tevR(n)%qq_hjWm(:,i) - tevR(n)%qq_hjWp(1,i))) )/tevR(n)%qq_hjWp(1,i)) .lt. 1.0D-5 )then
         simple_tevR=.True.
        else
         simple_tevR=.False.
        endif

        if(simple_tevR)then
          theo(n)%tev%XS_hjW_ratio(i)=tevR(n)%qq_hjWp(1,i)
        else
          theo(n)%tev%XS_hjW_ratio(i)=                          &
           &   sum( tevR(n)%qq_hjWp(:,i)*tevS(1)%qq_hjWp(:,i) ) &
           & + sum( tevR(n)%qq_hjWm(:,i)*tevS(1)%qq_hjWm(:,i) )
        endif
    
        theo(n)%tev%XS_hj_ratio(i)=                         &
         &        tevR(n)%gg_hj(i)    *tevS(1)%gg_hj(i)     &
         & + sum( tevR(n)%qq_hj(:,i)  *tevS(1)%qq_hj(:,i)   )   

   
        !this if loop is here to make sure tevR(n)%qq_hjZ(1,i).eq.0.0D0 is taken care of
        if(tevR(n)%qq_hjZ(1,i).eq.0.0D0)then
         simple_tevR=.False.
        elseif( ( (abs(sum( tevR(n)%qq_hjZ(:,i) - tevR(n)%qq_hjZ(1,i)))/tevR(n)%qq_hjZ(1,i)).lt. 1.0D-5) &
    &      .and. (               tevS(1)%gg_hjZ(i).le.0.0D0                  ) )then
         simple_tevR=.True.
        else
         simple_tevR=.False.
        endif

        if(  simple_tevR  )then    
          theo(n)%tev%XS_hjZ_ratio(i)=  tevR(n)%qq_hjZ(1,i)
        else
          theo(n)%tev%XS_hjZ_ratio(i)=                     &
           &        tevR(n)%gg_hjZ(i)   *tevS(1)%gg_hjZ(i)    &
           & + sum( tevR(n)%qq_hjZ(:,i) *tevS(1)%qq_hjZ(:,i)  )
        endif

        theo(n)%tev%XS_hjb_ratio(i)= tevR(n)%bg_hjb(i)
       else
        theo(n)%tev%XS_hjW_ratio(i)= 0.0D0       
        theo(n)%tev%XS_hj_ratio(i) = 0.0D0  
        theo(n)%tev%XS_hjZ_ratio(i)= 0.0D0
        theo(n)%tev%XS_hjb_ratio(i)= 0.0D0
       endif  
      enddo
     endif                     
    enddo            
    
    call deallocate_tevextras_parts(tevS)

 end subroutine XS_from_tevR
 !******************************************************************  
 subroutine fill_tevS(j,Mhj)
 ! fills the elements of tevS using the functions in module theory_tevSfunctions
 !************************************************************
  use theory_tevSfunctions
  implicit none
  !--------------------------------------internal
  integer :: j
  double precision :: Mhj
  !----------------------------------------------
  
  tevS(1)%qq_hjWp(1,j)=tev_rHWpm_udb(Mhj)
  tevS(1)%qq_hjWp(2,j)=tev_rHWpm_csb(Mhj) 
   
  tevS(1)%qq_hjWm(1,j)=tev_rHWpm_dub(Mhj)
  tevS(1)%qq_hjWm(2,j)=tev_rHWpm_scb(Mhj)
  
  tevS(1)%gg_hj(j)=tev_rH_gg(Mhj)
  
  tevS(1)%qq_hj(1,j)=0.0D0
  tevS(1)%qq_hj(2,j)=0.0D0
  tevS(1)%qq_hj(3,j)=0.0D0
  tevS(1)%qq_hj(4,j)=0.0D0
  tevS(1)%qq_hj(5,j)=tev_rH_bb(Mhj)
  
  tevS(1)%gg_hjZ(j)=0.0D0
  
  tevS(1)%qq_hjZ(1,j)=tev_rHZ_ddb(Mhj)
  tevS(1)%qq_hjZ(2,j)=tev_rHZ_uub(Mhj)
  tevS(1)%qq_hjZ(3,j)=tev_rHZ_ssb(Mhj)
  tevS(1)%qq_hjZ(4,j)=tev_rHZ_ccb(Mhj)
  tevS(1)%qq_hjZ(5,j)=tev_rHZ_bbb(Mhj)
      

  if(abs(sum(tevS(1)%qq_hjWp(:,j))+sum(tevS(1)%qq_hjWm(:,j)) - 1.0D0) .gt. 1.0D-2)then 
   stop 'error in fill_tevS (a)'
  elseif(abs(tevS(1)%gg_hj(j)+sum(tevS(1)%qq_hj(:,j)) - 1.0D0) .gt. 1.0D-2)then
   stop 'error in fill_tevS (b)'
  elseif(abs(tevS(1)%gg_hjZ(j)+sum(tevS(1)%qq_hjZ(:,j)) - 1.0D0) .gt. 1.0D-2)then
   stop 'error in fill_tevS (c)'
  endif 

 end subroutine fill_tevS 
 !******************************************************************  
end module theo_manip
!******************************************************************
